#! /usr/bin/perl -w
#====================================================================
# What's this ?
#====================================================================
# This plugin test the state of power supply's controller of SAN Storage.
# 
# Copyright (C) 2007 Djamal DJAE
# Copyright (C) 2007 LINAGORA
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# GPL Licence : http://www.gnu.org/licenses/gpl.txt
#
#====================================================================
# More contributions on http://www.linagora.org
#====================================================================

#====================================================================
# Changelog
#====================================================================
# Version 0.1 (2007/06):
# - First implementation
# Author: Djamal DJAE (LINAGORA)
#====================================================================

#====================================================================
# Modules
#====================================================================
use strict;
use lib '/usr/lib64/nagios/plugins/hitachi';
use utils qw /%ERRORS &print_revision &support/;
use Getopt::Long;
use vars qw($opt_V $opt_h $opt_v $opt_C $opt_H $opt_w $opt_c $verbose $opt_perfdata $opt_port $opt_disk);
use Net::SNMP;


my $VERSION = '0.1';



my $PROGNAME = $0;
sub print_help ();
sub print_usage ();

# get options
Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"         => \$opt_h,
     "d"   => \$verbose,'verbose'     => \$verbose,
     "V"   => \$opt_V, "version"      => \$opt_V,
     "v=s" => \$opt_v, "snmp=s"       => \$opt_v,
     "C=s" => \$opt_C, "community=s"  => \$opt_C,
     "H=s" => \$opt_H, "hostname=s"   => \$opt_H,
     "p=s" => \$opt_port, 'port=s'    => \$opt_port,
     "D=s"  => \$opt_disk, 'disk=s'    => \$opt_disk
     );

if ($verbose) {
        print "=> Verbose mode activated\n";
}

#====================================================================
# Version
#====================================================================
if ($opt_V) {
        &print_revision($PROGNAME, $VERSION);
        exit $ERRORS{'UNKNOWN'};
}

#====================================================================
# Help
#====================================================================
if ($opt_h) {
        print_help();
        exit $ERRORS{'OK'};
}



#====================================================================
# Configuration
#====================================================================
my $snmpdkcHWPS = '.1.3.6.1.4.1.116.5.11.4.1.1.6.1.6';
my $snmpdkuHWPS = '.1.3.6.1.4.1.116.5.11.4.1.1.7.1.2';
my %HWEnvironment = ( "1" => "noError", "2" => "acute", "3" => "serious",
		    "4" => "moderate", "5" => "service", );

# get SNMP version
($opt_v) || ($opt_v = "1");


# get SNMP community the default is public
($opt_C) || ($opt_C = "public");


# port default is 161
($opt_port) || ($opt_port = 161);


# SNMP Session
my ( $session, $error ) = Net::SNMP->session(-hostname  => $opt_H,-community => $opt_C, -version  => $opt_v, -port => $opt_port);
if ( !defined($session) ) {
    print("SESSION UNKNOWN: $error");
    exit $ERRORS{'UNKNOWN'};
}


# get the name of each descr
my $response = undef;

if (!defined ($response = $session->get_table($snmpdkcHWPS)) ||!defined ($response = $session->get_table($snmpdkuHWPS)) ) {
    my $answer=$session->error;
    $session->close;
    my $state = 'UNKNOWN';
    printf ("$state: SNMP error with snmp version $opt_v ($answer) when checking names\n");
    $session->close;
    exit $ERRORS{$state};
}

if ( $opt_disk == 'DKC'){
	foreach my $key ( keys %{$response}) {
		if ( $key =~ /\.(\d+)$/ ) {
	    		if ( ( $response->{$key} == 3 )  ){
        			print "CRITICAL: Power supply of DKC is  ".$HWEnvironment{$response->{$key}}."<br>";
        			exit $ERRORS{'CRITICAL'};
    			}
		}
	}
}
else{
	 foreach my $key ( keys %{$response}) {
             if ( $key =~ /\.(\d+)$/ ) {
                	if ( ( $response->{$key} == 3 ) ){
                        	print "CRITICAL: Power supply of DKU  is  ".$HWEnvironment{$response->{$key}}."<br>";
                        	exit $ERRORS{'CRITICAL'};
                	}
             }
	}
}


print "OK: No serious problem is  detected \n";
exit $ERRORS{'OK'};

sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   -H (--hostname)   Hostname to query - (required)\n";
    print "   -C (--community)  SNMP read community (defaults to public,\n";
    print "                     used with SNMP v1 and v2c\n";
    print "   -v (--snmp_version)  1 for SNMP v1 (default)\n";
    print "                        2 or 2c for SNMP v2c\n";
    print "   -D (--disk) DKU || DKC \n";
    print "   -V (--version)    Plugin version\n";
    print "   -h (--help)       usage help\n";
    print "   -d (--verbose)    verbose mode\n";
}

sub print_help () {
    print "Copyright (c) 2007 LINAGORA SA\n";
    print "Bugs to http://www.linagora.com/\n";
    print "\n";
    print_usage();
    print "\n";
}
